#!/bin/bash
#
# Installation Script for IBM JavaPOS 1.7.4
#   - untars the main ibm-javapos-1.7.4-for-linux-<version>.tgz
#   - next untars other files
#   - installs the javapos/javaxus/poss components
#   - compiles ibm kernel mode drives and copy
# 
# refer to: IBM JavaPOS 1.7.4 Installatin for Linux.pdf for details
#   - to install and uninstall individual javapos/javaxusb/poss comp.
#

# Defines
INSTALL_DIR="/tmp"
KERNEL_VERSION=""

JAVAPOS_INSTALL="ibm-javapos-1.7.5-for-linux"
JAVAPOS_TAR="ibmjavapos1.7.5"
JAVAXUSB_TAR="javaxusb"
POSS_TAR="ibmposs-linux-3.1.0"
POSS_KERNEL_TAR="ibmposs-linux-2.6-kernel-drivers"

DRIVER_DIR="2.6-kernel-drivers"
JAVAPOS_DIR="ibm-javapos"
POSS_DIR="ibmposs-linux-3.1.0"
JAVAX_USB_DIR="ibm-javaxusb"
DCS_DIR="dcs"
KBD_DIR="kbd"

KERNEL_VERSION=$(uname -r)
echo ${KERNEL_VERSION}

# untar main installation pakage
#
tar -zxvf ${JAVAPOS_INSTALL}*

# untar JavaPOS, Javax-usb, Poss 
#
tar -zxvf ${JAVAPOS_TAR}*
tar -zxvf ${JAVAXUSB_TAR}*
tar -zxvf ${POSS_TAR}*
tar -zxvf ${POSS_KERNEL_TAR}*

#install IBM JavaPOS
#
echo "Installing IBM JavaPOS..."
cd ${INSTALL_DIR}/${JAVAPOS_DIR}
chmod +x install_javapos.sh
./install_javapos.sh --install -v  2>/dev/null

#install javax-usb
#
echo "Installing javaxusb..."
cd ${INSTALL_DIR}/${JAVAX_USB_DIR}
chmod +x install_javaxusb.sh
./install_javaxusb.sh --install -v  2>/dev/null

#install ibmposs subsystem
#
echo "Installing IBM Poss subsystem..."
cd ${INSTALL_DIR}/${POSS_DIR}
chmod +x install_pos.sh
./install_pos.sh --install -v 2>/dev/null

#create misc directory for copying driver files
#
if [ ! -d /lib/modules/${KERNEL_VERSION}/misc ]; then
  mkdir /lib/modules/${KERNEL_VERSION}/misc
fi

#build drivers and copy
#
echo "Building kernel mode drivers..."
cd ${INSTALL_DIR}/${DRIVER_DIR}/${DCS_DIR}
make 2>/dev/null
echo "Copying kernel mode drivers..."
cp *.ko /lib/modules/${KERNEL_VERSION}/misc
cd ${INSTALL_DIR}/${DRIVER_DIR}/${KBD_DIR}
echo "Building keyboard kernel mode driver..."
make 2>/dev/null
echo "Copying keyboard kernel mode driver..."
cp *.ko /lib/modules/${KERNEL_VERSION}/misc
cd /lib/modules/${KERNEL_VERSION}/misc
echo "depmod -ae"
depmod -ae

